"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
;
;
var Utils = {
    bindMeteorEnvironment: Meteor.bindEnvironment.bind(Meteor),
    promisifyNodeCallback: function (fn) {
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return new Promise(function (resolve, reject) {
                fn.apply(void 0, args.concat([function (error, result) {
                        if (error) {
                            reject(error);
                            return;
                        }
                        resolve(result);
                    }]));
            });
        };
    },
    meteorAwaitPromise: function (promise) { return Utils.meteorWrapAsync(function (promise, callback) {
        promise.then(function (result) {
            callback(null, result);
        }, function (error) {
            callback(error);
        });
    })(promise); },
    meteorWrapAsync: Meteor.wrapAsync.bind(Meteor),
};
var Wrappers = {
    Collection: {
        find: Utils.promisifyNodeCallback(Utils.bindMeteorEnvironment(function (collection) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var callback = args.pop();
            var cursor;
            try {
                cursor = collection.find.apply(collection, args);
            }
            catch (error) {
                callback(error);
                return;
            }
            callback(null, cursor);
        })),
        findOne: rxjs_1.bindNodeCallback(Utils.bindMeteorEnvironment(function (collection) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var callback = args.pop();
            var document;
            try {
                document = collection.findOne.apply(collection, args);
            }
            catch (error) {
                callback(error);
                return;
            }
            callback(null, document);
        })),
        insert: rxjs_1.bindNodeCallback(Utils.bindMeteorEnvironment(function (collection, document, callback) {
            var _id;
            try {
                _id = collection.insert(document);
            }
            catch (error) {
                callback(error);
                return;
            }
            callback(null, _id);
        })),
        remove: rxjs_1.bindNodeCallback(Utils.bindMeteorEnvironment(function (collection, selector, callback) {
            var changes;
            try {
                changes = collection.remove(selector);
            }
            catch (error) {
                callback(error);
                return;
            }
            callback(null, changes);
        })),
        update: rxjs_1.bindNodeCallback(Utils.bindMeteorEnvironment(function (collection, selector, modifier) {
            var args = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                args[_i - 3] = arguments[_i];
            }
            var callback = args.pop();
            var numberAffected;
            try {
                numberAffected = collection.update.apply(collection, [selector, modifier].concat(args));
            }
            catch (error) {
                callback(error);
                return;
            }
            callback(null, numberAffected);
        })),
        upsert: rxjs_1.bindNodeCallback(Utils.bindMeteorEnvironment(function (collection, selector, modifier) {
            var args = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                args[_i - 3] = arguments[_i];
            }
            var callback = args.pop();
            var result;
            try {
                result = collection.upsert.apply(collection, [selector, modifier].concat(args));
            }
            catch (error) {
                callback(error);
                return;
            }
            callback(null, result);
        })),
    },
    Cursor: {
        count: rxjs_1.bindNodeCallback(Utils.bindMeteorEnvironment(function (cursorPromise) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var callback = args.pop();
            var count;
            try {
                var cursor = Utils.meteorAwaitPromise(cursorPromise);
                count = cursor.count.apply(cursor, args);
            }
            catch (error) {
                callback(error);
                return;
            }
            callback(null, count);
        })),
        fetch: rxjs_1.bindNodeCallback(Utils.bindMeteorEnvironment(function (cursorPromise, callback) {
            var result;
            try {
                var cursor = Utils.meteorAwaitPromise(cursorPromise);
                result = cursor.fetch();
            }
            catch (error) {
                callback(error);
                return;
            }
            callback(null, result);
        })),
        observe: rxjs_1.bindNodeCallback(Utils.bindMeteorEnvironment(function (cursorPromise, _a, callback) {
            var initial = _a.initial, added = _a.added, addedAt = _a.addedAt, changed = _a.changed, changedAt = _a.changedAt, removed = _a.removed, removedAt = _a.removedAt, movedTo = _a.movedTo;
            callback(null, rxjs_1.Observable.create(Utils.bindMeteorEnvironment(function (observer) {
                var handle;
                var initialDocuments = [];
                var finish$ = new rxjs_1.Subject();
                var observerProxy = {
                    error: function (error) {
                        observer.error(error);
                        finish$.next();
                    },
                    next: observer.next.bind(observer),
                };
                try {
                    var cursor = Utils.meteorAwaitPromise(cursorPromise);
                    handle = cursor.observe({
                        added: added || (initial && !addedAt)
                            ? function (document) {
                                if (initialDocuments) {
                                    initialDocuments.push(document);
                                }
                                else if (added) {
                                    rxjs_1.defer(function () {
                                        var result = added(document);
                                        return result
                                            ? rxjs_1.from(result)
                                            : rxjs_1.empty();
                                    }).pipe(operators_1.takeUntil(finish$)).subscribe(observerProxy);
                                }
                            }
                            : undefined,
                        addedAt: addedAt
                            ? function (document, atIndex, before) {
                                if (initialDocuments) {
                                    initialDocuments.splice(atIndex, 0, document);
                                }
                                else {
                                    rxjs_1.defer(function () {
                                        var result = addedAt(document, atIndex, before);
                                        return result
                                            ? rxjs_1.from(result)
                                            : rxjs_1.empty();
                                    }).pipe(operators_1.takeUntil(finish$)).subscribe(observerProxy);
                                }
                            }
                            : undefined,
                        changed: changed
                            ? function (newDocument, oldDocument) {
                                rxjs_1.defer(function () {
                                    var result = changed(newDocument, oldDocument);
                                    return result
                                        ? rxjs_1.from(result)
                                        : rxjs_1.empty();
                                }).pipe(operators_1.takeUntil(finish$)).subscribe(observerProxy);
                            }
                            : undefined,
                        changedAt: changedAt
                            ? function (newDocument, oldDocument, indexAt) {
                                rxjs_1.defer(function () {
                                    var result = changedAt(newDocument, oldDocument, indexAt);
                                    return result
                                        ? rxjs_1.from(result)
                                        : rxjs_1.empty();
                                }).pipe(operators_1.takeUntil(finish$)).subscribe(observerProxy);
                            }
                            : undefined,
                        removed: removed
                            ? function (oldDocument) {
                                rxjs_1.defer(function () {
                                    var result = removed(oldDocument);
                                    return result
                                        ? rxjs_1.from(result)
                                        : rxjs_1.empty();
                                }).pipe(operators_1.takeUntil(finish$)).subscribe(observerProxy);
                            }
                            : undefined,
                        removedAt: removedAt
                            ? function (oldDocument, atIndex) {
                                rxjs_1.defer(function () {
                                    var result = removedAt(oldDocument, atIndex);
                                    return result
                                        ? rxjs_1.from(result)
                                        : rxjs_1.empty();
                                }).pipe(operators_1.takeUntil(finish$))
                                    .subscribe(observerProxy);
                            }
                            : undefined,
                        movedTo: movedTo
                            ? function (document, fromIndex, toIndex, before) {
                                rxjs_1.defer(function () {
                                    var result = movedTo(document, fromIndex, toIndex, before);
                                    return result
                                        ? rxjs_1.from(result)
                                        : rxjs_1.empty();
                                }).pipe(operators_1.takeUntil(finish$)).subscribe(observerProxy);
                            }
                            : undefined,
                    });
                    if (initial) {
                        rxjs_1.of(initialDocuments).pipe(operators_1.switchMap(function (documents) {
                            var result = initial(documents);
                            return result
                                ? rxjs_1.from(result)
                                : rxjs_1.empty();
                        }), operators_1.takeUntil(finish$)).subscribe(observerProxy);
                    }
                    initialDocuments = null;
                }
                catch (error) {
                    observerProxy.error(error);
                }
                return function () {
                    handle.stop();
                    finish$.next();
                };
            })));
        })),
    },
};
var ObservableCursor = /** @class */ (function () {
    function ObservableCursor(cursor) {
        this.cursor = cursor;
        this.count = function countFn() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var _a;
            return (_a = Wrappers.Cursor).count.apply(_a, [this.cursor].concat(args));
        };
    }
    ObservableCursor.prototype.fetch = function () {
        return Wrappers.Cursor.fetch(this.cursor);
    };
    ;
    ObservableCursor.prototype.observe = function (callbacks) {
        return Wrappers.Cursor.observe(this.cursor, callbacks).pipe(operators_1.switchAll());
    };
    return ObservableCursor;
}());
exports.ObservableCursor = ObservableCursor;
var ObservableCollection = /** @class */ (function () {
    function ObservableCollection(collection) {
        this.collection = collection;
        this.find = function findFn() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var _a;
            return new ObservableCursor((_a = Wrappers.Collection).find.apply(_a, [this.collection].concat(args)));
        };
        this.findOne = function findOneFn() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var _a;
            return (_a = Wrappers.Collection).findOne.apply(_a, [this.collection].concat(args));
        };
        this.update = function updateFn(selector, modifier) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var _a;
            return (_a = Wrappers.Collection).update.apply(_a, [this.collection, selector, modifier].concat(args));
        };
        this.upsert = function upsertFn(selector, modifier) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var _a;
            return (_a = Wrappers.Collection).upsert.apply(_a, [this.collection, selector, modifier].concat(args));
        };
    }
    ObservableCollection.prototype.insert = function (document) {
        return Wrappers.Collection.insert(this.collection, document);
    };
    ObservableCollection.prototype.rawCollection = function () {
        return this.collection.rawCollection();
    };
    ObservableCollection.prototype.rawDatabase = function () {
        return this.collection.rawDatabase();
    };
    ObservableCollection.prototype.remove = function (selector) {
        return Wrappers.Collection.remove(this.collection, selector);
    };
    return ObservableCollection;
}());
exports.ObservableCollection = ObservableCollection;
//# sourceMappingURL=index.js.map